"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTimeTaken = exports.getTotalResults = exports.getLinkType = exports.getFirstMatch = exports.getUrlFromQuery = exports.getDomain = void 0;
const models_1 = require("./models");
const getDomain = (url, base) => {
    let href;
    try {
        href = new URL(url, base);
    }
    catch (error) {
        console.error(`🦠  url: ${JSON.stringify(url, null, 2)}`);
        console.error(`🦠  base: ${JSON.stringify(base, null, 2)}`);
        throw error;
    }
    ;
    return href.hostname;
};
exports.getDomain = getDomain;
const getUrlFromQuery = (query) => {
    const searchParams = new URLSearchParams(query.replace('/url?', ''));
    // if there is no q parameter, url is related to google search and we will return it in full
    return searchParams.get('q') || 'https://google.com' + query;
};
exports.getUrlFromQuery = getUrlFromQuery;
const getFirstMatch = (str, reg) => {
    str = str ? str : '';
    const matches = str.match(reg);
    return matches ? matches[0] : '';
};
exports.getFirstMatch = getFirstMatch;
const getLinkType = (url, base) => {
    const href = new URL(url, base);
    let linkType = models_1.LinkType.landing;
    if (href.pathname === '/') {
        linkType = models_1.LinkType.home;
    }
    else if (url.toLowerCase().includes('blog') ||
        url.toLowerCase().startsWith('https://medium.com/') ||
        url.toLowerCase().startsWith('https://nordicapis.com/') ||
        url.toLowerCase().startsWith('https://dev.to/')) {
        linkType = models_1.LinkType.blog;
    }
    return linkType;
};
exports.getLinkType = getLinkType;
const getTotalResults = (text) => {
    const resultsRegex = /[\d,]+(?= results)/g;
    const resultsMatched = exports.getFirstMatch(text, resultsRegex).replace(/,/g, '');
    return resultsMatched !== '' ? parseInt(resultsMatched, 10) : undefined;
};
exports.getTotalResults = getTotalResults;
const getTimeTaken = (text) => {
    const timeRegex = /[\d.]+(?= seconds)/g;
    const timeMatched = exports.getFirstMatch(text, timeRegex);
    return timeMatched !== '' ? parseFloat(timeMatched) : undefined;
};
exports.getTimeTaken = getTimeTaken;
